unit LEConverter;

{$ifdef FPC}{$mode OBJFPC}{$endif}

interface
  procedure LEConvertCharTo(const value: char; const offset: longint; var data: array of byte);
  procedure LEConvertInt16To(const value: smallint; const offset: longint; var data: array of byte);
  procedure LEConvertUInt16To(const value: word; const offset: longint; var data: array of byte);
  procedure LEConvertInt32To(const value: longint; const offset: longint; var data: array of byte);
  procedure LEConvertUInt32To(const value: longword; const offset: longint; var data: array of byte);
  procedure LEConvertUInt64To(const value: qword; const offset: longint; var data: array of byte);
  procedure LEConvertBooleanTo(const value: boolean; const offset: longint; var data: array of byte);

  function LEConvertInt16From(const offset: longint; const data: array of byte): smallint;
  function LEConvertUInt16From(const offset: longint; const data: array of byte): word;
  function LEConvertInt32From(const offset: longint; const data: array of byte): longint;
  function LEConvertUInt32From(const offset: longint; const data: array of byte): longword;
  function LEConvertUInt64From(const offset: longint; const data: array of byte): qword;
  function LEConvertStringFrom(const offset: longint; const len: longint; const data: array of byte): string;
  function LEConvertBooleanFrom(const offset: longint; const data: array of byte): boolean;

type
  TByteArray = array of byte;
  PByteArray = ^TByteArray;

implementation

procedure LEConvertCharTo(const value: char; const offset: longint; var data: array of byte);
begin
  data[offset + 0] := byte(value);
end;

procedure LEConvertInt16To(const value: smallint; const offset: longint; var data: array of byte);
begin
  data[offset + 0] := byte(value);
  data[offset + 1] := byte((value shr 8) and $FF);
end;

procedure LEConvertUInt16To(const value: word; const offset: longint; var data: array of byte);
begin
  data[offset + 0] := byte(value);
  data[offset + 1] := byte((value shr 8) and $FF);
end;

procedure LEConvertInt32To(const value: longint; const offset: longint; var data: array of byte);
begin
  data[offset + 0] := byte(value);
  data[offset + 1] := byte((value shr  8) and $FF);
  data[offset + 2] := byte((value shr 16) and $FF);
  data[offset + 3] := byte((value shr 24) and $FF);
end;

procedure LEConvertUInt32To(const value: longword; const offset: longint; var data: array of byte);
begin
  data[offset + 0] := byte(value);
  data[offset + 1] := byte((value shr  8) and $FF);
  data[offset + 2] := byte((value shr 16) and $FF);
  data[offset + 3] := byte((value shr 24) and $FF);
end;

procedure LEConvertUInt64To(const value: qword; const offset: longint; var data: array of byte);
begin
  data[offset + 0] := byte(value);
  data[offset + 1] := byte((value shr  8) and $FF);
  data[offset + 2] := byte((value shr 16) and $FF);
  data[offset + 3] := byte((value shr 24) and $FF);
  data[offset + 4] := byte((value shr 32) and $FF);
  data[offset + 5] := byte((value shr 40) and $FF);
  data[offset + 6] := byte((value shr 48) and $FF);
  data[offset + 7] := byte((value shr 56) and $FF);
end;

procedure LEConvertBooleanTo(const value: boolean; const offset: longint; var data: array of byte);
begin
  if (value) then begin
    data[offset + 0] := 1;
  end
  else begin
    data[offset + 0] := 0;
  end;
end;

function LEConvertInt16From(const offset: longint; const data: array of byte): smallint;
begin
  result := word(data[offset + 0]) shl 0 or
            word(data[offset + 1]) shl 8;
end;

function LEConvertUInt16From(const offset: longint; const data: array of byte): word;
begin
  result := word(data[offset + 0]) shl 0 or
            word(data[offset + 1]) shl 8;
end;

function LEConvertInt32From(const offset: longint; const data: array of byte): longint;
begin
  result := longint(data[offset + 0]) shl  0 or
            longint(data[offset + 1]) shl  8 or
            longint(data[offset + 2]) shl 16 or
            longint(data[offset + 3]) shl 24;
end;

function LEConvertUInt32From(const offset: longint; const data: array of byte): longword;
begin
  result := longint(data[offset + 0]) shl  0 or
            longint(data[offset + 1]) shl  8 or
            longint(data[offset + 2]) shl 16 or
            longint(data[offset + 3]) shl 24;
end;

function LEConvertUInt64From(const offset: longint; const data: array of byte): qword;
begin
  result := qword(data[offset + 0]) shl  0 or
            qword(data[offset + 1]) shl  8 or
            qword(data[offset + 2]) shl 16 or
            qword(data[offset + 3]) shl 24 or
            qword(data[offset + 4]) shl 32 or
            qword(data[offset + 5]) shl 40 or
            qword(data[offset + 6]) shl 48 or
            qword(data[offset + 7]) shl 56;
end;

function LEConvertStringFrom(const offset: longint; const len: longint; const data: array of byte): string;
var i: longint;
begin
  result := '';
  for i := offset to (offset + len) do begin
    if (data[i] = 0) then break;
    result := result + char(data[i]);
  end;
end;

function LEConvertBooleanFrom(const offset: longint; const data: array of byte): boolean;
begin
  result := data[offset] <> 0;
end;

end.
